; $id$
;
; License: GPL-3

(declare (export))
(use svnwiki-extensions-support iconv stream-ext srfi-1 srfi-40 orders format-modular html-stream posix)

(define *languages*
  '((es "Español")
    (en "English")
    (fr "Français")
    (pt "Português")))

(define (translations-links env)
  (let-from-environment env (path-out-real path-in params)
    (let* ((components (string-split path-out-real "/"))
           (special (svnwiki-is-special? #f path-out-real))
           (base-filename
             (string-append
               (stream->string (svnwiki-file-without-language (last components)))
               "."))
           (dir
             (svnwiki-make-pathname
               (cons path-in
                     ((if special butlast identity)
                      (butlast components)))))
           (translations
             (stream-filter
               (lambda (file)
                 (and (> (string-length file) (string-length base-filename))
                      (substring=? base-filename file)
                      (or (not (string=? file path-out-real))
                          special)))
               (if (directory? dir)
                 (list->stream (directory dir))
                 stream-null)))
           (charset (svnwiki-encoding-for-file env))
           (charset-convert
             (if (string=? charset "utf-8")
               identity
               (let ((descriptor (iconv-open charset "utf-8")))
                 (cut iconv descriptor <>)))))
      (if (stream-null? translations)
        (html-stream
          (p (cdr (or (assoc 'none params) (cons #f (html-stream "No translations are available for this file."))))))
        (html-stream
          (ul
            (stream-concatenate
              (stream-map
                (lambda (file)
                  (html-stream
                    ((li class "translationLink")
                     ((a href
                         (svnwiki-make-pathname
                           (and special "..")
                           file
                           (mime->ending
                             ((handler-mime (get-handler (environment env ((path path-out-real)))))
                              path-in
                              path-out-real))))
                      (let* ((language (svnwiki-file-language file))
                             (name (and language (assoc (stream->symbol language) *languages*))))
                        (if name
                          (charset-convert (cadr name))
                          language))))))
                translations))))))))

(svnwiki-extension-define 'code-break 'translations translations-links)
